﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Microsoft.Research.Kinect.Nui;

using System.Runtime.InteropServices;
using System.Drawing.Imaging;
namespace fullskeleton
{
    public partial class Form1 : Form
    {
        Runtime nui;
        PlanarImage videoimage;


        public Form1()
        {
            InitializeComponent();
            nui = Runtime.Kinects[0];
        }

        private void button1_Click(object sender, EventArgs e)
        {
            nui.Initialize(RuntimeOptions.UseDepthAndPlayerIndex | RuntimeOptions.UseSkeletalTracking | RuntimeOptions.UseColor);
            nui.VideoStream.Open(ImageStreamType.Video, 2, ImageResolution.Resolution640x480, ImageType.Color);
            //     nui.DepthStream.Open(ImageStreamType.Depth, 2, ImageResolution.Resolution320x240, ImageType.DepthAndPlayerIndex);
            nui.SkeletonFrameReady += new EventHandler<SkeletonFrameReadyEventArgs>(SkeletonFrameReady);
            nui.VideoFrameReady += new EventHandler<ImageFrameReadyEventArgs>(FrameReady);
        }

        void SkeletonFrameReady(object sender, SkeletonFrameReadyEventArgs e)
        {
            if (videoimage.Bits == null) return;
            
            SkeletonFrame skeletonFrame = e.SkeletonFrame;
            Bitmap bmap = PImageToBitmap(videoimage);
            Graphics g = Graphics.FromImage(bmap);
           
            foreach (SkeletonData data in skeletonFrame.Skeletons)
            {
                if (data.TrackingState == SkeletonTrackingState.Tracked)
                {
                    //body
                    DrawBone(JointID.Head, JointID.ShoulderCenter, data, g);
                    DrawBone(JointID.ShoulderCenter, JointID.Spine, data, g);
                    DrawBone(JointID.Spine, JointID.HipCenter, data, g);
                    //left leg
                    DrawBone(JointID.HipCenter, JointID.HipLeft, data, g);
                    DrawBone(JointID.HipLeft, JointID.KneeLeft, data, g);
                    DrawBone(JointID.KneeLeft, JointID.AnkleLeft, data, g);
                    DrawBone(JointID.AnkleLeft, JointID.FootLeft, data, g);
                    //Right Leg
                    DrawBone(JointID.HipCenter, JointID.HipRight, data, g);
                    DrawBone(JointID.HipRight, JointID.KneeRight, data, g);
                    DrawBone(JointID.KneeRight, JointID.AnkleRight, data, g);
                    DrawBone(JointID.AnkleRight, JointID.FootRight, data, g);
                    //Left Arm
                    DrawBone(JointID.ShoulderCenter, JointID.ShoulderLeft, data, g);
                    DrawBone(JointID.ShoulderLeft, JointID.ElbowLeft, data, g);
                    DrawBone(JointID.ElbowLeft, JointID.WristLeft, data, g);
                    DrawBone(JointID.WristLeft, JointID.HandLeft, data, g);
                    //Right Arm
                    DrawBone(JointID.ShoulderCenter, JointID.ShoulderRight, data, g);
                    DrawBone(JointID.ShoulderRight, JointID.ElbowRight, data, g);
                    DrawBone(JointID.ElbowRight, JointID.WristRight, data, g);
                    DrawBone(JointID.WristRight, JointID.HandRight, data, g);

                }



            }
            pictureBox1.Image = bmap;
        }

        void DrawBone(JointID j1, JointID j2, SkeletonData data, Graphics g)
        {
            Point p1 = GetJoint(j1, data);
            Point p2 = GetJoint(j2, data);
            g.DrawLine(Pens.Red, p1, p2);
        }

        Point GetJoint(JointID j, SkeletonData data)
        {
            Vector location = data.Joints[j].Position;
            float xd, yd;
            nui.SkeletonEngine.SkeletonToDepthImage(location, out xd, out yd);
            xd = Math.Max(0, Math.Min(xd * 320, 320));
            yd = Math.Max(0, Math.Min(yd * 240, 240));
            int x, y;
            ImageViewArea iv = new ImageViewArea();
            nui.NuiCamera.GetColorPixelCoordinatesFromDepthPixel(ImageResolution.Resolution640x480, iv, (int)xd, (int)yd, (short)0, out x, out y);
            return new Point(x, y);
        }

        void FrameReady(object sender, ImageFrameReadyEventArgs e)
        {
            videoimage = e.ImageFrame.Image;
        }

 
        Bitmap PImageToBitmap(PlanarImage PImage)
        {
            Bitmap bmap = new Bitmap(
               PImage.Width,
               PImage.Height,
               PixelFormat.Format32bppRgb);
            BitmapData bmapdata = bmap.LockBits(
                new Rectangle(0, 0, PImage.Width,
                PImage.Height),
                ImageLockMode.WriteOnly,
                bmap.PixelFormat);
            IntPtr ptr = bmapdata.Scan0;
            Marshal.Copy(PImage.Bits,
                         0,
                         ptr,
                         PImage.Width *
                         PImage.BytesPerPixel *
                         PImage.Height);
            bmap.UnlockBits(bmapdata);
            return bmap;
        }
       
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            nui.Uninitialize();
        }

    }

}
